@@ -7,7 +7,6 @@ from typing import (
     Union,
 )
 
-import aiofiles
 import aiohttp
 import pydantic
 import ujson
@@ -19,6 +18,11 @@ from .errors import (
     ResponseParseError,
     errors_classes,
 )
+from .responses import (
+    PydanticModelResponseClass,
+    ResponseClass,
+    StreamResponseClass,
+)
 from .utils import (
     DEFAULT_DOWNLOAD_CHUNK_SIZE,
     model_to_dict,
@@ -33,7 +37,7 @@ Headers = Union[StrIntMapping, pydantic.BaseModel]
 Body = Union[dict[str, Any], pydantic.BaseModel]
 ErrorResponseModels = dict[int, Type[pydantic.BaseModel]]
 
-Response = TypeVar('Response')
+ResponseType = TypeVar('ResponseType')
 
 
 class Client:
@@ -43,12 +47,16 @@ class Client:
             *,
             headers: Headers = None,
             cookies: Cookies = None,
+            params: Params = None,
             error_response_models: ErrorResponseModels = None,
-            bearer_token: Union[str, pydantic.SecretStr] = None
+            bearer_token: Union[str, pydantic.SecretStr] = None,
+            response_class: Type[ResponseClass] = PydanticModelResponseClass,
+            error_response_class: Type[ResponseClass] = PydanticModelResponseClass,
     ):
         self.logger = logging.getLogger("pydantic_aiohttp.Client")
         headers = model_to_dict(headers) or {}
         cookies = model_to_dict(cookies) or {}
+        params = model_to_dict(params) or {}
 
         if bearer_token is not None:
             if isinstance(bearer_token, pydantic.SecretStr):
@@ -57,6 +65,9 @@ class Client:
             headers["Authorization"] = f"Bearer {bearer_token}"
 
         self._error_response_models = error_response_models or {}
+        self._response_class = response_class
+        self._error_response_class = error_response_class
+        self._params = params
         self._session = aiohttp.ClientSession(
             base_url,
             headers=headers,
@@ -67,7 +78,8 @@ class Client:
     async def _parse_response_error(
             self,
             response: aiohttp.ClientResponse,
-            error_response_models: ErrorResponseModels = None
+            error_response_models: ErrorResponseModels = None,
+            error_response_class: Type[ResponseClass] = PydanticModelResponseClass,
     ):
         error_response_models = self._error_response_models | (error_response_models or {})
         error_class = errors_classes.get(response.status, HTTPError)
@@ -84,22 +96,6 @@ class Client:
 
         raise error_class(response_json)
 
-    async def _parse_response_json(
-            self,
-            response: aiohttp.ClientResponse,
-            response_model: Type[Response] = None,
-    ):
-        try:
-            response_json = await response.json(loads=ujson.loads, content_type=None)
-        except JSONDecodeError:
-            response_text = await response.text()
-            raise ResponseParseError(raw_response=response_text)
-
-        if response_model is not None:
-            return pydantic.parse_obj_as(response_model, response_json)
-
-        return response_json
-
     async def download_file(
             self,
             path: str,
@@ -108,34 +104,23 @@ class Client:
             headers: Headers = None,
             cookies: Cookies = None,
             params: Params = None,
-            response_model: Type[Response] = None,
             timeout: int = 300,  # Default in aiohttp
             error_response_models: ErrorResponseModels = None,
             chunk_size: int = DEFAULT_DOWNLOAD_CHUNK_SIZE,
-    ):
-        async with self._session.request(
-                'GET',
-                path,
-                headers=model_to_dict(headers),
-                cookies=model_to_dict(cookies),
-                params=model_to_dict(params),
-                timeout=aiohttp.ClientTimeout(total=timeout)
-        ) as response:
-            if response.ok:
-                if response.content_type == 'application/json':
-                    return await self._parse_response_json(response, response_model=response_model)
-
-                if filepath is None:
-                    # TODO: Retrieve original file name from response
-                    filepath = path.rsplit("/", maxsplit=1)[-1]
-
-                async with aiofiles.open(filepath, 'wb') as fd:
-                    async for chunk in response.content.iter_chunked(chunk_size):
-                        await fd.write(chunk)
-
-                return
-
-            return await self._parse_response_error(response, error_response_models=error_response_models)
+    ) -> aiohttp.typedefs.PathLike:
+        return await self.request(
+            'GET',
+            path,
+            headers=headers,
+            cookies=cookies,
+            params=params,
+            timeout=timeout,
+            response_class=StreamResponseClass,
+            error_response_models=error_response_models,
+            # Response parse kwargs
+            filepath=filepath,
+            chunk_size=chunk_size
+        )
 
     async def upload_file(
             self,
@@ -146,10 +131,10 @@ class Client:
             headers: Headers = None,
             cookies: Cookies = None,
             params: Params = None,
-            response_model: Type[Response] = None,
+            response_model: Type[ResponseType] = None,
             timeout: int = 300,  # Default in aiohttp
             error_response_models: ErrorResponseModels = None,
-    ) -> Optional[Response]:
+    ) -> Optional[ResponseType]:
         return await self.post(
             path,
             headers=headers,
@@ -170,10 +155,10 @@ class Client:
             headers: Headers = None,
             cookies: Cookies = None,
             params: Params = None,
-            response_model: Type[Response] = None,
+            response_model: Type[ResponseType] = None,
             timeout: int = 300,  # Default in aiohttp
             error_response_models: ErrorResponseModels = None
-    ) -> Optional[Response]:
+    ) -> Optional[ResponseType]:
         return await self.post(
             path,
             headers=headers,
@@ -195,22 +180,37 @@ class Client:
             headers: Headers = None,
             cookies: Cookies = None,
             params: Params = None,
-            response_model: Type[Response] = None,
+            response_model: Type[ResponseType] = None,
             timeout: int = 300,  # Default in aiohttp
-            error_response_models: ErrorResponseModels = None
-    ) -> Optional[Response]:
+            error_response_models: ErrorResponseModels = None,
+            response_class: Type[ResponseClass] = None,
+            **response_class_parse_kwargs
+    ) -> Optional[ResponseType]:
+        response_class = response_class or self._response_class
+
+        if not bool(response_class):
+            raise ValueError('response_class is not set')
+
+        _params = self._params
+
+        if bool(params):
+            _params.update(model_to_dict(params) or {})
+
         async with self._session.request(
                 method,
                 path,
                 headers=model_to_dict(headers),
                 cookies=model_to_dict(cookies),
-                params=model_to_dict(params),
+                params=_params,
                 json=model_to_dict(body),
                 data=data,
                 timeout=aiohttp.ClientTimeout(total=timeout)
         ) as response:
             if response.ok:
-                return await self._parse_response_json(response, response_model=response_model)
+                return await response_class(response).parse(
+                    response_model=response_model,
+                    **response_class_parse_kwargs
+                )
 
             return await self._parse_response_error(response, error_response_models=error_response_models)
 
@@ -221,10 +221,11 @@ class Client:
             headers: Headers = None,
             cookies: Cookies = None,
             params: Params = None,
-            response_model: Type[Response] = None,
+            response_model: Type[ResponseType] = None,
             timeout: int = 300,  # Default in aiohttp
-            error_response_models: ErrorResponseModels = None
-    ) -> Optional[Response]:
+            error_response_models: ErrorResponseModels = None,
+            response_class: Type[ResponseClass] = None
+    ) -> Optional[ResponseType]:
         return await self.request(
             "GET",
             path,
@@ -234,6 +235,7 @@ class Client:
             response_model=response_model,
             timeout=timeout,
             error_response_models=error_response_models,
+            response_class=response_class
         )
 
     async def post(
@@ -245,10 +247,11 @@ class Client:
             headers: Headers = None,
             cookies: Cookies = None,
             params: Params = None,
-            response_model: Type[Response] = None,
+            response_model: Type[ResponseType] = None,
             timeout: int = 300,  # Default in aiohttp
-            error_response_models: ErrorResponseModels = None
-    ) -> Optional[Response]:
+            error_response_models: ErrorResponseModels = None,
+            response_class: Type[ResponseClass] = None
+    ) -> Optional[ResponseType]:
         return await self.request(
             "POST",
             path,
@@ -260,6 +263,7 @@ class Client:
             response_model=response_model,
             timeout=timeout,
             error_response_models=error_response_models,
+            response_class=response_class
         )
 
     async def patch(
@@ -271,10 +275,11 @@ class Client:
             headers: Headers = None,
             cookies: Cookies = None,
             params: Params = None,
-            response_model: Type[Response] = None,
+            response_model: Type[ResponseType] = None,
             timeout: int = 300,  # Default in aiohttp
-            error_response_models: ErrorResponseModels = None
-    ) -> Optional[Response]:
+            error_response_models: ErrorResponseModels = None,
+            response_class: Type[ResponseClass] = None
+    ) -> Optional[ResponseType]:
         return await self.request(
             "PATCH",
             path,
@@ -286,6 +291,7 @@ class Client:
             response_model=response_model,
             timeout=timeout,
             error_response_models=error_response_models,
+            response_class=response_class
         )
 
     async def put(
@@ -297,10 +303,11 @@ class Client:
             headers: Headers = None,
             cookies: Cookies = None,
             params: Params = None,
-            response_model: Type[Response] = None,
+            response_model: Type[ResponseType] = None,
             timeout: int = 300,  # Default in aiohttp
-            error_response_models: ErrorResponseModels = None
-    ) -> Optional[Response]:
+            error_response_models: ErrorResponseModels = None,
+            response_class: Type[ResponseClass] = None
+    ) -> Optional[ResponseType]:
         return await self.request(
             "PUT",
             path,
@@ -312,6 +319,7 @@ class Client:
             response_model=response_model,
             timeout=timeout,
             error_response_models=error_response_models,
+            response_class=response_class
         )
 
     async def delete(
@@ -323,10 +331,11 @@ class Client:
             headers: Headers = None,
             cookies: Cookies = None,
             params: Params = None,
-            response_model: Type[Response] = None,
+            response_model: Type[ResponseType] = None,
             timeout: int = 300,  # Default in aiohttp
-            error_response_models: ErrorResponseModels = None
-    ) -> Optional[Response]:
+            error_response_models: ErrorResponseModels = None,
+            response_class: Type[ResponseClass] = None
+    ) -> Optional[ResponseType]:
         return await self.request(
             "DELETE",
             path,
@@ -338,7 +347,15 @@ class Client:
             response_model=response_model,
             timeout=timeout,
             error_response_models=error_response_models,
+            response_class=response_class
         )
 
     async def close(self):
         await self._session.close()
+
+    async def __aenter__(self):
+        return self
+
+    async def __aexit__(self, exc_type, exc_val, exc_tb):
+        await self.close()
+        return self
